@@ -1,11 +1,11 @@
 //!
-//! NOTE: Documents => [MapxRawVs](crate::versioned::mapx_raw)
+//! Documents => [MapxRawVs](crate::versioned::mapx_raw)
 //!
 
 use crate::{
     common::ende::{KeyEnDe, ValueEnDe},
     versioned::mapx_ord_rawkey::{MapxOrdRawKeyVs, MapxOrdRawKeyVsIter},
-    BranchName, ParentBranchName, VerChecksum, VersionName,
+    BranchName, ParentBranchName, VersionName,
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
@@ -14,6 +14,7 @@ use std::{
     ops::{Bound, RangeBounds},
 };
 
+/// Documents => [MapxRawVs](crate::versioned::mapx_raw::MapxRawVs)
 #[derive(Clone, Serialize, Deserialize, PartialEq, Eq, Debug)]
 #[serde(bound = "")]
 pub struct MapxVs<K, V>
@@ -377,7 +378,7 @@ where
         )
     }
 
-    crate::impl_vcs_methods!();
+    crate::impl_vs_methods!();
 }
 
 pub struct MapxVsIter<'a, K, V>
