@@ -1,17 +1,16 @@
 //!
-//! NOTE: Documents => [MapxRawVs](crate::versioned::mapx_raw)
+//! Documents => [MapxRawVs](crate::versioned::mapx_raw)
 //!
 
 use crate::{
-    common::{
-        ende::ValueEnDe, BranchName, ParentBranchName, RawKey, VerChecksum, VersionName,
-    },
+    common::{ende::ValueEnDe, BranchName, ParentBranchName, RawKey, VersionName},
     versioned::mapx_raw::{MapxRawVs, MapxRawVsIter},
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
 use std::{marker::PhantomData, ops::RangeBounds};
 
+/// Documents => [MapxRawVs](crate::versioned::mapx_raw::MapxRawVs)
 #[derive(Clone, Serialize, Deserialize, PartialEq, Eq, Debug)]
 #[serde(bound = "")]
 pub struct MapxOrdRawKeyVs<V>
@@ -392,7 +391,7 @@ where
         self.inner.clear();
     }
 
-    crate::impl_vcs_methods!();
+    crate::impl_vs_methods!();
 }
 
 pub struct MapxOrdRawKeyVsIter<'a, V>
@@ -429,13 +428,16 @@ where
 impl<'a, V> ExactSizeIterator for MapxOrdRawKeyVsIter<'a, V> where V: ValueEnDe {}
 
 #[macro_export(crate)]
-macro_rules! impl_vcs_methods {
+macro_rules! impl_vs_methods {
     () => {
+        /// Create a new version on the default branch.
         #[inline(always)]
         pub fn version_create(&mut self, version_name: VersionName) -> Result<()> {
             self.inner.version_create(version_name).c(d!())
         }
 
+        /// Create a new version on a specified branch,
+        /// NOTE: the branch must has been created.
         #[inline(always)]
         pub fn version_create_by_branch(
             &mut self,
@@ -447,11 +449,13 @@ macro_rules! impl_vcs_methods {
                 .c(d!())
         }
 
+        /// Check if a verison exists on default branch.
         #[inline(always)]
         pub fn version_exists(&self, version_name: VersionName) -> bool {
             self.inner.version_exists(version_name)
         }
 
+        /// Check if a version exists on a specified branch(include its parents).
         #[inline(always)]
         pub fn version_exists_on_branch(
             &self,
@@ -462,11 +466,13 @@ macro_rules! impl_vcs_methods {
                 .version_exists_on_branch(version_name, branch_name)
         }
 
+        /// Check if a version is directly created on the default branch.
         #[inline(always)]
         pub fn version_created(&self, version_name: VersionName) -> bool {
             self.inner.version_created(version_name)
         }
 
+        /// Check if a version is directly created on a specified branch(exclude its parents).
         #[inline(always)]
         pub fn version_created_on_branch(
             &self,
@@ -477,21 +483,37 @@ macro_rules! impl_vcs_methods {
                 .version_created_on_branch(version_name, branch_name)
         }
 
+        /// Remove the newest version on the default branch.
+        ///
+        /// 'Write'-like operations on branches and versions are different from operations on data.
+        ///
+        /// 'Write'-like operations on data require recursive tracing of all parent nodes,
+        /// while operations on branches and versions are limited to their own perspective,
+        /// and should not do any tracing.
         #[inline(always)]
         pub fn version_pop(&mut self) -> Result<()> {
             self.inner.version_pop().c(d!())
         }
 
+        /// Remove the newest version on a specified branch.
+        ///
+        /// 'Write'-like operations on branches and versions are different from operations on data.
+        ///
+        /// 'Write'-like operations on data require recursive tracing of all parent nodes,
+        /// while operations on branches and versions are limited to their own perspective,
+        /// and should not do any tracing.
         #[inline(always)]
         pub fn version_pop_by_branch(&mut self, branch_name: BranchName) -> Result<()> {
             self.inner.version_pop_by_branch(branch_name).c(d!())
         }
 
+        /// Create a new branch based on the head of the default branch.
         #[inline(always)]
         pub fn branch_create(&mut self, branch_name: BranchName) -> Result<()> {
             self.inner.branch_create(branch_name).c(d!())
         }
 
+        /// Create a new branch based on the head of a specified branch.
         #[inline(always)]
         pub fn branch_create_by_base_branch(
             &mut self,
@@ -503,21 +525,60 @@ macro_rules! impl_vcs_methods {
                 .c(d!())
         }
 
+        /// Create a new branch based on a specified version of a specified branch.
+        #[inline(always)]
+        pub fn branch_create_by_base_branch_version(
+            &mut self,
+            branch_name: BranchName,
+            base_branch_name: ParentBranchName,
+            base_version_name: VersionName,
+        ) -> Result<()> {
+            self.inner
+                .branch_create_by_base_branch_version(
+                    branch_name,
+                    base_branch_name,
+                    base_version_name,
+                )
+                .c(d!())
+        }
+
+        /// Check if a branch exists or not.
         #[inline(always)]
         pub fn branch_exists(&self, branch_name: BranchName) -> bool {
             self.inner.branch_exists(branch_name)
         }
 
+        /// Remove a branch, remove all changes directly made by this branch.
+        ///
+        /// 'Write'-like operations on branches and versions are different from operations on data.
+        ///
+        /// 'Write'-like operations on data require recursive tracing of all parent nodes,
+        /// while operations on branches and versions are limited to their own perspective,
+        /// and should not do any tracing.
         #[inline(always)]
         pub fn branch_remove(&mut self, branch_name: BranchName) -> Result<()> {
             self.inner.branch_remove(branch_name).c(d!())
         }
 
+        /// Remove all changes directly made by versions(bigger than `last_version_id`) of this branch.
+        ///
+        /// 'Write'-like operations on branches and versions are different from operations on data.
+        ///
+        /// 'Write'-like operations on data require recursive tracing of all parent nodes,
+        /// while operations on branches and versions are limited to their own perspective,
+        /// and should not do any tracing.
         #[inline(always)]
         pub fn branch_truncate(&mut self, branch_name: BranchName) -> Result<()> {
             self.inner.branch_truncate(branch_name).c(d!())
         }
 
+        /// Remove all changes directly made by versions(bigger than `last_version_id`) of this branch.
+        ///
+        /// 'Write'-like operations on branches and versions are different from operations on data.
+        ///
+        /// 'Write'-like operations on data require recursive tracing of all parent nodes,
+        /// while operations on branches and versions are limited to their own perspective,
+        /// and should not do any tracing.
         #[inline(always)]
         pub fn branch_truncate_to(
             &mut self,
@@ -529,54 +590,44 @@ macro_rules! impl_vcs_methods {
                 .c(d!())
         }
 
+        /// Remove the newest version on a specified branch.
+        ///
+        /// 'Write'-like operations on branches and versions are different from operations on data.
+        ///
+        /// 'Write'-like operations on data require recursive tracing of all parent nodes,
+        /// while operations on branches and versions are limited to their own perspective,
+        /// and should not do any tracing.
         #[inline(always)]
         pub fn branch_pop_version(&mut self, branch_name: BranchName) -> Result<()> {
             self.inner.branch_pop_version(branch_name).c(d!())
         }
 
+        /// Merge a branch to its parent branch.
         #[inline(always)]
         pub fn branch_merge_to_parent(&mut self, branch_name: BranchName) -> Result<()> {
             self.inner.branch_merge_to_parent(branch_name).c(d!())
         }
 
+        /// Check if a branch has children branches.
         #[inline(always)]
         pub fn branch_has_children(&self, branch_name: BranchName) -> bool {
             self.inner.branch_has_children(branch_name)
         }
 
+        /// Make a branch to be default,
+        /// all default operations will be applied to it.
         #[inline(always)]
         pub fn branch_set_default(&mut self, branch_name: BranchName) -> Result<()> {
             self.inner.branch_set_default(branch_name).c(d!())
         }
 
-        #[inline(always)]
-        pub fn checksum_get(&self) -> Option<VerChecksum> {
-            self.inner.checksum_get()
-        }
-
-        #[inline(always)]
-        pub fn checksum_get_by_branch(
-            &self,
-            branch_name: BranchName,
-        ) -> Option<VerChecksum> {
-            self.inner.checksum_get_by_branch(branch_name)
-        }
-
-        #[inline(always)]
-        pub fn checksum_get_by_branch_version(
-            &self,
-            branch_name: BranchName,
-            version_name: VersionName,
-        ) -> Option<VerChecksum> {
-            self.inner
-                .checksum_get_by_branch_version(branch_name, version_name)
-        }
-
+        /// Clean outdated versions out of the default reserved number.
         #[inline(always)]
         pub fn prune(&mut self, reserved_ver_num: Option<usize>) -> Result<()> {
             self.inner.prune(reserved_ver_num).c(d!())
         }
 
+        /// Clean outdated versions out of a specified reserved number.
         #[inline(always)]
         pub fn prune_by_branch(
             &mut self,
