@@ -42,8 +42,8 @@ mod backend;
 mod test;
 
 use crate::common::{
-    BranchName, ParentBranchName, RawKey, RawValue, VerChecksum, VersionName,
-    INITIAL_BRANCH_NAME, NULL,
+    BranchName, ParentBranchName, RawKey, RawValue, VersionName, INITIAL_BRANCH_NAME,
+    NULL,
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
@@ -444,7 +444,7 @@ impl MapxRawVs {
             .and_then(|br_id| {
                 self.inner
                     .get_version_id(branch_name, version_name)
-                    .map(|ver_id| self.inner.version_exists_on_branch(ver_id, br_id))
+                    .map(|ver_id| self.inner.version_exists_on_branch(ver_id, br_id).0)
             })
             .unwrap_or(false)
     }
@@ -522,6 +522,27 @@ impl MapxRawVs {
             })
     }
 
+    /// Create a new branch based on a specified version of a specified branch.
+    #[inline(always)]
+    pub fn branch_create_by_base_branch_version(
+        &mut self,
+        branch_name: BranchName,
+        base_branch_name: ParentBranchName,
+        base_version_name: VersionName,
+    ) -> Result<()> {
+        let base_br_id = self
+            .inner
+            .get_branch_id(BranchName(base_branch_name.0))
+            .c(d!("base branch not found"))?;
+        let base_ver_id = self
+            .inner
+            .get_version_id(BranchName(base_branch_name.0), base_version_name)
+            .c(d!("base vesion not found"))?;
+        self.inner
+            .branch_create_by_base_branch_version(branch_name.0, base_br_id, base_ver_id)
+            .c(d!())
+    }
+
     /// Check if a branch exists or not.
     #[inline(always)]
     pub fn branch_exists(&self, branch_name: BranchName) -> bool {
@@ -631,36 +652,6 @@ impl MapxRawVs {
             .and_then(|brid| self.inner.branch_set_default(brid).c(d!()))
     }
 
-    /// Get the signature of the head of the default branch.
-    #[inline(always)]
-    pub fn checksum_get(&self) -> Option<VerChecksum> {
-        self.inner.checksum_get()
-    }
-
-    /// Get the signature of the head of a specified branch.
-    #[inline(always)]
-    pub fn checksum_get_by_branch(
-        &self,
-        branch_name: BranchName,
-    ) -> Option<VerChecksum> {
-        self.inner
-            .get_branch_id(branch_name)
-            .and_then(|id| self.inner.checksum_get_by_branch(id))
-    }
-
-    /// Get the signature of a specified version of a specified branch.
-    #[inline(always)]
-    pub fn checksum_get_by_branch_version(
-        &self,
-        branch_name: BranchName,
-        version_name: VersionName,
-    ) -> Option<VerChecksum> {
-        let br_id = self.inner.get_branch_id(branch_name)?;
-        let ver_id = self.inner.get_version_id(branch_name, version_name)?;
-        self.inner
-            .checksum_get_by_branch_version(br_id, Some(ver_id))
-    }
-
     /// Clean outdated versions out of the default reserved number.
     #[inline(always)]
     pub fn prune(&mut self, reserved_ver_num: Option<usize>) -> Result<()> {
