@@ -1,6 +1,6 @@
 use super::*;
 use crate::{
-    common::{BranchName, ParentBranchName, VersionName, BRANCH_CNT_LIMIT},
+    common::{BranchName, ParentBranchName, VersionName, BRANCH_ANCESTORS_LIMIT},
     ValueEnDe,
 };
 use std::{sync::mpsc::channel, thread};
@@ -93,8 +93,6 @@ fn VCS_mgmt() {
 // - can not read data created by newer versions from an older version
 // - remove a non-existing key will sucess
 // - data created by a version will disappear after the version has been removed
-// - insert same values for a same key within one version will not change the version checksum
-// - insert different values for a same key within one version will change the version checksum
 // - can not remove a version except it is the HEAD version(no public functions privided)
 // - can not remove a KV except its version is the HEAD version(no public functions privided)
 // - can not write data to a version except it is the HEAD version(no public functions privided)
@@ -103,40 +101,22 @@ fn version_operations(hdr: &mut MapxRawVs) {
     assert!(hdr.insert(b"key", b"value").is_err());
 
     hdr.version_create(VersionName(b"v-001")).unwrap();
-    let checksum_v_001_initial = hdr.checksum_get().unwrap();
 
     // use existing version name
     assert!(hdr.version_create(VersionName(b"v-001")).is_err());
-    assert_eq!(checksum_v_001_initial, hdr.checksum_get().unwrap());
 
     assert!(hdr.is_empty());
 
     hdr.version_create(VersionName(b"v-002")).unwrap();
 
-    let checksum_v_002 = hdr.checksum_get().unwrap();
-    assert!(checksum_v_001_initial != checksum_v_002);
-    assert_eq!(
-        checksum_v_001_initial,
-        hdr.checksum_get_by_branch_version(BranchName(b"main"), VersionName(b"v-001"))
-            .unwrap()
-    );
-
     assert!(hdr.is_empty());
     assert!(hdr.is_empty_by_branch(BranchName(b"main")));
     assert!(hdr.is_empty_by_branch_version(BranchName(b"main"), VersionName(b"v-001")));
     assert!(hdr.is_empty_by_branch_version(BranchName(b"main"), VersionName(b"v-002")));
 
-    assert!(hdr.insert(b"v-002/key-01", b"v-002/value-01").is_ok());
-    let checksum_v_002_01 = hdr.checksum_get().unwrap();
-    assert!(checksum_v_002 != checksum_v_002_01);
-
-    assert!(hdr.insert(b"v-002/key-02", b"v-002/value-02").is_ok());
-    let checksum_v_002_02 = hdr.checksum_get().unwrap();
-    assert!(checksum_v_002_01 != checksum_v_002_02);
-
-    // insert a same key-value will not change the checksum value
-    assert!(hdr.insert(b"v-002/key-02", b"v-002/value-02").is_ok());
-    assert_eq!(checksum_v_002_02, hdr.checksum_get().unwrap());
+    pnk!(hdr.insert(b"v-002/key-01", b"v-002/value-01"));
+    pnk!(hdr.insert(b"v-002/key-02", b"v-002/value-02"));
+    pnk!(hdr.insert(b"v-002/key-02", b"v-002/value-02"));
 
     assert_eq!(
         hdr.get(b"v-002/key-01"),
@@ -177,7 +157,7 @@ fn version_operations(hdr: &mut MapxRawVs) {
     assert!(!hdr.is_empty_by_branch_version(BranchName(b"main"), VersionName(b"v-002")));
 
     hdr.version_create(VersionName(b"v-003")).unwrap();
-    assert!(hdr.insert(b"v-003/key-01", b"v-003/value-01").is_ok());
+    pnk!(hdr.insert(b"v-003/key-01", b"v-003/value-01"));
     assert_eq!(&hdr.get(b"v-003/key-01").unwrap()[..], b"v-003/value-01");
 
     assert_eq!(hdr.remove(b"fake key").unwrap(), None);
@@ -218,7 +198,7 @@ fn version_operations(hdr: &mut MapxRawVs) {
     assert!(!hdr.is_empty_by_branch_version(BranchName(b"main"), VersionName(b"v-002")));
     assert!(!hdr.is_empty_by_branch_version(BranchName(b"main"), VersionName(b"v-003")));
 
-    assert!(hdr.insert(b"v-003/key-02", b"v-003/value-02").is_ok());
+    pnk!(hdr.insert(b"v-003/key-02", b"v-003/value-02"));
     assert!(
         hdr.get_by_branch_version(
             b"v-003/key-02",
@@ -257,7 +237,7 @@ fn version_operations(hdr: &mut MapxRawVs) {
     );
 
     // update a existing key
-    assert!(hdr.insert(b"v-002/key-02", b"v-002/value-03").is_ok());
+    pnk!(hdr.insert(b"v-002/key-02", b"v-002/value-03"));
     // get the new view of this value
     assert_eq!(
         &hdr.get_by_branch_version(
@@ -269,7 +249,7 @@ fn version_operations(hdr: &mut MapxRawVs) {
         b"v-002/value-03"
     );
 
-    assert!(hdr.version_pop().is_ok());
+    pnk!(hdr.version_pop());
     assert!(
         hdr.get_by_branch_version(
             b"v-003/key-02",
@@ -319,7 +299,6 @@ fn version_operations(hdr: &mut MapxRawVs) {
 // - a branch can only be merged to its parent branch
 // - every branch with a same parent can be merged to their parent branch
 //     - all verisons will be ordered by the inner-defined version id
-// - version checksums except the latest version will not be changed
 fn branch_operations(hdr: &mut MapxRawVs) {
     hdr.branch_create(BranchName(b"b-1")).unwrap();
     hdr.branch_create(BranchName(b"b-2")).unwrap();
@@ -349,17 +328,11 @@ fn branch_operations(hdr: &mut MapxRawVs) {
     //
     // Although "v-001" is not unique globally("v-001" exists on "main" branch),
     // but it is unique in the "b-1" branch, so we can use it.
-    assert!(
-        hdr.version_create_by_branch(VersionName(b"v-001"), BranchName(b"b-1"))
-            .is_ok()
-    );
+    pnk!(hdr.version_create_by_branch(VersionName(b"v-001"), BranchName(b"b-1")));
 
     hdr.version_create_by_branch(VersionName(b"v-004"), BranchName(b"b-1"))
         .unwrap();
-    assert!(
-        hdr.insert_by_branch(b"v-001/key-01", b"v-001/value-01", BranchName(b"b-1"))
-            .is_ok()
-    );
+    pnk!(hdr.insert_by_branch(b"v-001/key-01", b"v-001/value-01", BranchName(b"b-1")));
 
     // multi-layers view
     hdr.branch_create_by_base_branch(BranchName(b"b-1-child"), ParentBranchName(b"b-1"))
@@ -391,19 +364,52 @@ fn branch_operations(hdr: &mut MapxRawVs) {
         .is_none()
     );
 
-    // branch number limits
-    (4..BRANCH_CNT_LIMIT).for_each(|i| {
-        assert!(hdr.branch_create(BranchName(&i.to_be_bytes())).is_ok());
+    // total number of branch has no limits
+    (4..2 * BRANCH_ANCESTORS_LIMIT).for_each(|i| {
+        pnk!(hdr.branch_create(BranchName(&i.to_be_bytes())));
     });
-    assert!(
-        hdr.branch_create(BranchName(&1usize.to_be_bytes()))
-            .is_err()
-    );
-    (4..BRANCH_CNT_LIMIT).for_each(|i| {
-        assert!(hdr.branch_remove(BranchName(&i.to_be_bytes())).is_ok());
+    (4..2 * BRANCH_ANCESTORS_LIMIT).for_each(|i| {
+        pnk!(hdr.branch_remove(BranchName(&i.to_be_bytes())));
     });
-    assert!(hdr.branch_create(BranchName(&1usize.to_be_bytes())).is_ok());
-    assert!(hdr.branch_remove(BranchName(&1usize.to_be_bytes())).is_ok());
+
+    pnk!(hdr.branch_create_by_base_branch_version(
+        BranchName(&3usize.to_be_bytes()),
+        ParentBranchName(b"main"),
+        VersionName(b"v-002")
+    ));
+    pnk!(hdr.version_create_by_branch(
+        VersionName(b"verN"),
+        BranchName(&3usize.to_be_bytes())
+    ));
+    assert!(hdr.branch_has_children(BranchName(b"main")));
+    assert!(!hdr.branch_has_children(BranchName(&3usize.to_be_bytes())));
+
+    // total number of branch ancestor has limits
+    (4..(BRANCH_ANCESTORS_LIMIT - 2)).for_each(|i| {
+        pnk!(hdr.branch_create_by_base_branch(
+            BranchName(&i.to_be_bytes()),
+            ParentBranchName(&(i - 1).to_be_bytes())
+        ));
+        pnk!(hdr.version_create_by_branch(
+            VersionName(b"verN"),
+            BranchName(&i.to_be_bytes())
+        ));
+    });
+    pnk!(hdr.branch_create_by_base_branch(
+        BranchName(&(BRANCH_ANCESTORS_LIMIT - 2).to_be_bytes()),
+        ParentBranchName(&(BRANCH_ANCESTORS_LIMIT - 3).to_be_bytes()),
+    ));
+    (3..(BRANCH_ANCESTORS_LIMIT - 2)).for_each(|i| {
+        // can not remove because they all have one child
+        assert!(hdr.branch_has_children(BranchName(&i.to_be_bytes())));
+        assert!(hdr.branch_remove(BranchName(&i.to_be_bytes())).is_err());
+    });
+    (3..(BRANCH_ANCESTORS_LIMIT - 1)).rev().for_each(|i| {
+        pnk!(hdr.branch_remove(BranchName(&i.to_be_bytes())));
+    });
+
+    pnk!(hdr.branch_create(BranchName(&1usize.to_be_bytes())));
+    pnk!(hdr.branch_remove(BranchName(&1usize.to_be_bytes())));
 
     // not exist
     assert!(hdr.branch_remove(BranchName(b"fake branch")).is_err());
@@ -414,9 +420,9 @@ fn branch_operations(hdr: &mut MapxRawVs) {
     assert!(hdr.branch_remove(BranchName(b"main")).is_err());
 
     // remove its children
-    assert!(hdr.branch_remove(BranchName(b"b-1-child")).is_ok());
+    pnk!(hdr.branch_remove(BranchName(b"b-1-child")));
     // not it can be removed
-    assert!(hdr.branch_remove(BranchName(b"b-1")).is_ok());
+    pnk!(hdr.branch_remove(BranchName(b"b-1")));
     assert!(
         hdr.get_by_branch(b"v-001/key-01", BranchName(b"b-1"))
             .is_none()
@@ -426,10 +432,11 @@ fn branch_operations(hdr: &mut MapxRawVs) {
     (0..100u64).for_each(|i| {
         hdr.version_create_by_branch(VersionName(&i.to_be_bytes()), BranchName(b"b-2"))
             .unwrap();
-        assert!(
-            hdr.insert_by_branch(&i.to_be_bytes(), &i.to_be_bytes(), BranchName(b"b-2"))
-                .is_ok()
-        );
+        pnk!(hdr.insert_by_branch(
+            &i.to_be_bytes(),
+            &i.to_be_bytes(),
+            BranchName(b"b-2")
+        ));
     });
 
     // verion names already exist
@@ -461,13 +468,10 @@ fn branch_operations(hdr: &mut MapxRawVs) {
 
     // now we can use these version names again
     (11..=100u64).for_each(|i| {
-        assert!(
-            hdr.version_create_by_branch(
-                VersionName(&i.to_be_bytes()),
-                BranchName(b"b-2")
-            )
-            .is_ok()
-        );
+        pnk!(hdr.version_create_by_branch(
+            VersionName(&i.to_be_bytes()),
+            BranchName(b"b-2")
+        ));
     });
 
     // get very old value after passing through many verions
@@ -494,7 +498,6 @@ fn branch_operations(hdr: &mut MapxRawVs) {
         .unwrap();
 
     // created data to be merged
-    let mut checksums = vec![];
     (0..10u64).for_each(|i| {
         hdr.branch_create_by_base_branch(
             BranchName(&i.to_be_bytes()),
@@ -507,30 +510,19 @@ fn branch_operations(hdr: &mut MapxRawVs) {
                 BranchName(&i.to_be_bytes()),
             )
             .unwrap();
-            assert!(
-                hdr.insert_by_branch(
-                    &((1 + i) * j).to_be_bytes(),
-                    &((1 + i) * j).to_be_bytes(),
-                    BranchName(&i.to_be_bytes())
-                )
-                .is_ok()
-            );
-            checksums.push(
-                hdr.checksum_get_by_branch(BranchName(&i.to_be_bytes()))
-                    .unwrap(),
-            );
+            pnk!(hdr.insert_by_branch(
+                &((1 + i) * j).to_be_bytes(),
+                &((1 + i) * j).to_be_bytes(),
+                BranchName(&i.to_be_bytes())
+            ));
         });
     });
 
     (0..10u64).for_each(|i| {
-        assert!(
-            hdr.branch_merge_to_parent(BranchName(&i.to_be_bytes()))
-                .is_ok()
-        );
+        pnk!(hdr.branch_merge_to_parent(BranchName(&i.to_be_bytes())));
     });
 
     // All versions and their chanegs are belong to the base branch now
-    let mut checksums_after_merge = vec![];
     (0..10u64).for_each(|i| {
         (1000..1010u64).for_each(|j| {
             // children branches have been removed
@@ -552,30 +544,8 @@ fn branch_operations(hdr: &mut MapxRawVs) {
                 .unwrap()[..],
                 &((1 + i) * j).to_be_bytes()
             );
-            checksums_after_merge.push(
-                hdr.checksum_get_by_branch_version(
-                    BranchName(b"b-2"),
-                    VersionName(&((1 + i) * j).to_be_bytes()),
-                )
-                .unwrap(),
-            );
         });
     });
-
-    // the checksum of the latest version will be changed,
-    // all other versions will keep their original checksum
-    let latest_checksum = *checksums_after_merge.last().unwrap();
-    assert_eq!(checksums.len(), checksums_after_merge.len());
-    // merged 10 times
-    assert_eq!(
-        10,
-        checksums
-            .iter()
-            .zip(checksums_after_merge.iter())
-            .filter(|(a, b)| a != b)
-            .count()
-    );
-    assert!(checksums.binary_search(&latest_checksum).is_err());
 }
 
 // prune version:
@@ -602,14 +572,11 @@ fn prune_operations(hdr: &mut MapxRawVs) {
                 BranchName(b"b-2"),
             )
             .unwrap();
-            assert!(
-                hdr.insert_by_branch(
-                    &((1 + i) * j).to_be_bytes(),
-                    &[0],
-                    BranchName(b"b-2")
-                )
-                .is_ok()
-            );
+            pnk!(hdr.insert_by_branch(
+                &((1 + i) * j).to_be_bytes(),
+                &[0],
+                BranchName(b"b-2")
+            ));
         });
     });
 
