@@ -1,16 +1,18 @@
 //!
-//! NOTE: Documents => [MapxRaw](crate::versioned::mapx_raw)
+//! Documents => [MapxRawVs](crate::versioned::mapx_raw)
 //!
 
 use crate::{
     versioned::mapx_ord_rawkey::MapxOrdRawKeyVs, BranchName, ParentBranchName,
-    ValueEnDe, VerChecksum, VersionName,
+    ValueEnDe, VersionName,
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
 
 /// Used to express some 'non-collection' types,
 /// such as any type of integer, an enum value, etc..
+///
+/// Documents => [MapxRawVs](crate::versioned::mapx_raw::MapxRawVs)
 #[derive(Clone, Serialize, Deserialize, Debug)]
 #[serde(bound = "")]
 pub struct OrphanVs<T>
@@ -25,9 +27,9 @@ where
     T: ValueEnDe,
 {
     #[inline(always)]
-    pub fn new(v: T, version_name: VersionName) -> Self {
+    pub fn new(v: T) -> Self {
         let mut hdr = MapxOrdRawKeyVs::new();
-        pnk!(hdr.version_create(version_name));
+        pnk!(hdr.version_create(VersionName(&[])));
         pnk!(hdr.insert_ref(&[], &v));
         Self { inner: hdr }
     }
@@ -80,5 +82,5 @@ where
             .get_by_branch_version(&[], branch_name, version_name)
     }
 
-    crate::impl_vcs_methods!();
+    crate::impl_vs_methods!();
 }
