@@ -1,14 +1,15 @@
 //!
-//! NOTE: Documents => [MapxRaw](crate::versioned::mapx_raw)
+//! Documents => [MapxRawVs](crate::versioned::mapx_raw)
 //!
 
 use crate::{
     versioned::mapx_ord_rawkey::{MapxOrdRawKeyVs, MapxOrdRawKeyVsIter},
-    BranchName, ParentBranchName, ValueEnDe, VerChecksum, VersionName,
+    BranchName, ParentBranchName, ValueEnDe, VersionName,
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
 
+/// Documents => [MapxRawVs](crate::versioned::mapx_raw::MapxRawVs)
 #[derive(Clone, Serialize, Deserialize, PartialEq, Eq, Debug)]
 #[serde(bound = "")]
 pub struct VecxVs<T: ValueEnDe> {
@@ -239,7 +240,7 @@ impl<T: ValueEnDe> VecxVs<T> {
         }
     }
 
-    crate::impl_vcs_methods!();
+    crate::impl_vs_methods!();
 }
 
 pub struct VecxVsIter<'a, T: ValueEnDe> {
